@@ -1,5 +1,6 @@
 setClass("AllModelClass",
          representation(pars='list', 
+                        model='list',
                         K='numeric', 
                         G2='numeric', 
                         df='numeric', 
@@ -28,6 +29,7 @@ setClass("AllModelClass",
                         nfact='numeric',                        
                         prodlist='list',
                         constrain='list',
+                        parprior='list',
                         fulldata='matrix',
                         information='matrix',
                         longpars='numeric',
