@@ -8,6 +8,7 @@ setClass("GroupPars",
                         nfact='numeric',
                         gradient='numeric',
                         hessian='matrix',                        
+                        itemtrace='matrix',
                         lbound='numeric',
                         ubound='numeric',
                         n.prior.mu='numeric',
@@ -28,7 +29,8 @@ setClass("AllItemsClass",
                         ncat='numeric',
                         rs='matrix',
                         gradient='numeric',
-                        hessian='matrix',                        
+                        hessian='matrix', 
+                        itemtrace='matrix',
                         lbound='numeric',
                         ubound='numeric',
                         n.prior.mu='numeric',
@@ -75,4 +77,6 @@ setGeneric('calcLogLik', function(object, ...) standardGeneric("calcLogLik"))
 
 setGeneric("itemplot.internal",  function(object, ...) standardGeneric("itemplot.internal"))
 
-setGeneric("fscores.internal", function(object, ...) standardGeneric("fscores.internal"))
\ No newline at end of file
+setGeneric("fscores.internal", function(object, ...) standardGeneric("fscores.internal"))
+
+setGeneric("summary", function(object, ...) standardGeneric("summary"))
\ No newline at end of file
