@@ -231,14 +231,15 @@ setMethod(
     f = "plot",
     signature = signature(x = 'ExploratoryClass', y = 'missing'),
     definition = function(x, y, type = 'info', npts = 50, theta_angle = 45, 
-                          rot = list(xaxis = -70, yaxis = 30, zaxis = 10), ...)
+                          rot = list(xaxis = -70, yaxis = 30, zaxis = 10), 
+                          auto.key = TRUE, ...)
     {           
-        if (!type %in% c('info','infocontour', 'SE')) stop(type, " is not a valid plot type.")
+        if (!type %in% c('info','infocontour', 'SE', 'trace')) stop(type, " is not a valid plot type.")
         if (any(theta_angle > 90 | theta_angle < 0)) 
             stop('Improper angle specifed. Must be between 0 and 90.')
         if(length(theta_angle) > 1) type = 'infoangle'
         rot <- list(x = rot[[1]], y = rot[[2]], z = rot[[3]])       
-        nfact <- x@pars[[1]]@nfact
+        nfact <- x@nfact
         if(nfact > 2) stop("Can't plot high dimensional solutions.")
         if(nfact == 1) theta_angle <- 0        
         J <- length(x@pars) - 1        
@@ -254,7 +255,7 @@ setMethod(
             for(i in 1:J)
                 info <- info + iteminfo(x=x@pars[[i]], Theta=ThetaFull, degrees=ta)            
         }
-        plt <- data.frame(cbind(info,Theta))         
+        plt <- data.frame(cbind(info,Theta=Theta))         
         if(nfact == 2){						
             colnames(plt) <- c("info", "Theta1", "Theta2")			
             plt$SE <- 1 / sqrt(plt$info)
@@ -274,7 +275,7 @@ setMethod(
                 return(wireframe(SE ~ Theta1 + Theta2, data = plt, main = "Test Standard Errors", 
                                  zlab=expression(SE(theta)), xlab=expression(theta[1]), ylab=expression(theta[2]), 
                                  scales = list(arrows = FALSE), screen = rot, colorkey = TRUE, drape = TRUE))            
-        } else {
+        } else {            
             colnames(plt) <- c("info", "Theta")
             plt$SE <- 1 / sqrt(plt$info)
             if(type == 'info')
@@ -283,8 +284,20 @@ setMethod(
             if(type == 'infocontour') 
                 cat('No \'contour\' plots for 1-dimensional models\n')
             if(type == 'SE')                
-                xyplot(SE~Theta, plt, type='l',main = 'Test Standard Errors', 
-                       xlab = expression(theta), ylab=expression(SE(theta)))
+                return(xyplot(SE~Theta, plt, type='l',main = 'Test Standard Errors', 
+                       xlab = expression(theta), ylab=expression(SE(theta))))
+            if(type == 'trace'){                
+                if(!all(x@K == 2)) stop('trace line plot only available for tests 
+                                        with dichotomous items')                
+                P <- matrix(NA, nrow(Theta), J)
+                for(i in 1:J)
+                    P[,i] <- probtrace(extract.item(x, i), ThetaFull)[,2]
+                items <- gl(n=J, k=nrow(Theta), labels = paste('Item', 1:J))
+                plotobj <- data.frame(P = as.numeric(P), Theta=Theta, item=items)
+                return(xyplot(P ~ Theta, plotobj, group = item, ylim = c(0,1), 
+                       xlab = expression(theta), ylab = expression(P(theta)), 
+                       auto.key = auto.key, type = 'l', main = 'Item trace lines', ...))            
+            }
         }		
     }		
 )	
