@@ -6,6 +6,8 @@ test_that('dich', {
     expect_is(modm1, 'ConfirmatoryClass')          
     modm2 <- mirt(data, 2, SE = TRUE)
     expect_is(modm2, 'ExploratoryClass')
+    modm2 <- mirt(data, 2, SE = TRUE, SE.type = 'BL')
+    expect_is(modm2, 'ExploratoryClass')
     modm3 <- mirt(data, 1, itemtype = 'Rasch', SE = FALSE)
     expect_is(modm3, 'ConfirmatoryClass')
     modm4 <- mirt(data, 1, itemtype = '1PL')    
@@ -28,6 +30,8 @@ test_that('dich', {
     expect_is(fm4, 'matrix')
     fm5 <- fscores(modm6, method = 'ML', full.scores = FALSE, verbose = FALSE)
     expect_is(fm5, 'matrix')
+    fm6 <- fscores(modm1, method = 'EAPsum', full.scores = FALSE, verbose = FALSE)
+    expect_is(fm6, 'data.frame')
     
     res1 <- residuals(modm1, verbose = FALSE)
     res2 <- residuals(modm2, verbose = FALSE)
@@ -35,7 +39,7 @@ test_that('dich', {
     expect_is(res2, 'matrix')
     IP1 <- itemplot(modm1, 1)
     IP2 <- itemplot(modm2, 1)
-    expect_is(IP1, 'NULL')
+    expect_is(IP1, 'trellis')
     expect_is(IP2, 'trellis')
     TP1 <- plot(modm1)
     TP2 <- plot(modm2)
