@@ -16,29 +16,38 @@ RcppExport SEXP Estep(SEXP Ritemtrace, SEXP Rprior, SEXP RX,
     */
     
     NumericVector prior(Rprior);
+    NumericVector log_prior(prior.length());
     IntegerVector nfact(Rnfact);
     IntegerVector r(Rr);
     IntegerMatrix data(RX);
     NumericMatrix itemtrace(Ritemtrace);
+    NumericMatrix log_itemtrace(itemtrace.nrow(), itemtrace.ncol());    
     int nquad = prior.length();
     int nitems = data.ncol();
     int npat = r.length();      
     double expd = 0.0;
-    int i = 0, k = 0, item = 0;	
+    int i = 0, k = 0, item = 0;    
     NumericMatrix r1(nquad, nitems);    
     NumericVector expected(npat), posterior(nquad);
     List ret;
+    for (item = 0; item < nitems; item++)
+        for (k = 0; k < nquad; k++)
+            log_itemtrace(k,item) = log(itemtrace(k,item));
+    for (k = 0; k < nquad; k++)
+        log_prior(k) = log(prior(k));
 	
     // Begin main function body 				
 	for (int pat = 0; pat < npat; pat++){		
   	    for (k = 0; k < nquad; k++)
-		    posterior(k) = prior(k);
+		    posterior(k) = 0.0;
 			
 		for (item = 0; item < nitems; item++){
             if (data(pat,item))
                 for (k = 0; k < nquad; k++)
-			        posterior(k) = posterior(k) * itemtrace(k,item);
+			        posterior(k) += log_itemtrace(k,item);
 		}    
+        for (i = 0; i < nquad; i++)
+            posterior(i) = exp(log_prior(i) + posterior(i));
 	    expd = 0.0;
 	    for (i = 0; i < nquad; i++)
 	        expd += posterior(i);	
@@ -52,7 +61,7 @@ RcppExport SEXP Estep(SEXP Ritemtrace, SEXP Rprior, SEXP RX,
 	                r1(k,item) += posterior(k);
 		    			
 	    }
-	} //end main 		
+	} //end main  		
  
     //return R list of length 3 with list("r1","r0","expected") 
     ret["r1"] = r1;
@@ -76,7 +85,9 @@ RcppExport SEXP Estepbfactor(SEXP Ritemtrace, SEXP Rprior, SEXP RX, SEXP Rr, SEX
         Rsitems = integer matrix. Specific factor indicator
     */
     NumericMatrix itemtrace(Ritemtrace);
+    NumericMatrix log_itemtrace(itemtrace.nrow(), itemtrace.ncol()); 
     NumericVector prior(Rprior);
+    NumericVector log_prior(prior.length());
     IntegerVector r(Rr);
     IntegerMatrix data(RX);
     IntegerMatrix sitems(Rsitems);
@@ -86,7 +97,12 @@ RcppExport SEXP Estepbfactor(SEXP Ritemtrace, SEXP Rprior, SEXP RX, SEXP Rr, SEX
     int npquad = prior.length();
     int nquad = npquad * npquad;  
     int npat = r.length();      
-    int i=0, j=0, k=0, item=0, fact=0;	          
+    int i=0, j=0, k=0, item=0, fact=0;
+    for (item = 0; item < nitems; item++)
+        for (k = 0; k < nquad; k++)
+            log_itemtrace(k,item) = log(itemtrace(k,item));
+    for (k = 0; k < npquad; k++)
+        log_prior(k) = log(prior(k));
         
 	//declare dependent arrays 
 	NumericVector tempsum(npquad), expected(npat), Pls(npquad);
@@ -97,14 +113,14 @@ RcppExport SEXP Estepbfactor(SEXP Ritemtrace, SEXP Rprior, SEXP RX, SEXP Rr, SEX
 	for (int pat = 0; pat < npat; pat++){		
 		for (fact = 0; fact < sfact; fact++){ 	
 			for (k = 0; k < nquad; k++)
-				likelihoods(k,fact) = 1;				
+				likelihoods(k,fact) = 0;				
 			for (item = 0; item < nitems; item++){
 				if (data(pat,item))	
 					if(sitems(item,fact))
 					    for (k = 0; k < nquad; k++)
-					  	    likelihoods(k,fact) = likelihoods(k,fact) * itemtrace(k,item);					
+					  	    likelihoods(k,fact) += log_itemtrace(k,item);					
 			}
-		}			
+		}         					
 		for (fact = 0; fact < sfact; fact++){
 			k = 0;
 			for (j = 0; j < npquad; j++){
@@ -116,13 +132,16 @@ RcppExport SEXP Estepbfactor(SEXP Ritemtrace, SEXP Rprior, SEXP RX, SEXP Rr, SEX
 			}
 			for (j = 0; j < npquad; j++)				
 			    for (i = 0; i < npquad; i++)
-			  	    L(i,j) = L(i,j) * prior(j);
+			  	    L(i,j) = exp(L(i,j) + log_prior(j));
 			for (j = 0; j < npquad; j++)				
 		        for (i = 0; i < npquad; i++)
 			        tempsum(j) += L(j,i);
 			for (i = 0; i < npquad; i++)
 			    Plk(i,fact) = tempsum(i);    			
-		}				
+		}		
+        for (fact = 0; fact < sfact; fact++)
+            for (k = 0; k < nquad; k++)
+    	        likelihoods(k,fact) = exp(likelihoods(k,fact));
 		expected(pat) = 0.0;
 		for (i = 0; i < npquad; i++){
 		    Pls(i) = 1.0; 		  		
@@ -155,3 +174,68 @@ RcppExport SEXP Estepbfactor(SEXP Ritemtrace, SEXP Rprior, SEXP RX, SEXP Rr, SEX
     END_RCPP
 }
 
+//EAP estimates used in multipleGroup
+RcppExport SEXP EAPgroup(SEXP Rlog_itemtrace, SEXP Rtabdata, SEXP RTheta, SEXP Rprior, SEXP Rmu) 
+{
+    BEGIN_RCPP
+
+    NumericMatrix log_itemtrace(Rlog_itemtrace); 
+    NumericMatrix tabdata(Rtabdata); 
+    NumericMatrix Theta(RTheta); 
+    NumericVector prior(Rprior);
+    NumericVector mu(Rmu);
+    int i, j, k, ind;
+    int n = prior.length(); //nquad
+    int nitems = tabdata.ncol();
+    int nfact = Theta.ncol();
+
+    NumericVector L(n), thetas(nfact), thetas2(nfact*(nfact+1)/2); 
+    NumericMatrix scores(tabdata.nrow(), nfact), scores2(tabdata.nrow(), nfact*(nfact + 1)/2);
+    double LL, denom;
+
+    for(int pat = 0; pat < tabdata.nrow(); pat++){
+        
+        for(j = 0; j < n; j++){
+            LL = 0.0;
+            for(i = 0; i < nitems; i++)
+               LL += tabdata(pat, i) * log_itemtrace(j, i); 
+            L(j) = exp(LL);
+        }
+        
+        thetas.fill(0.0);
+        denom = 0.0;
+        for(j = 0; j < n; j++)
+            denom += (L(j)*prior(j));
+        
+        for(k = 0; k < nfact; k++){            
+            for(j = 0; j < n; j++)
+                thetas(k) += Theta(j, k) * L(j) * prior(j) / denom;
+            scores(pat, k) = thetas(k);
+        }
+
+        ind = 0;
+        thetas2.fill(0.0);
+        for(i = 0; i < nfact; i++){
+            for(k = 0; k < nfact; k++){
+                if(i <= k){
+                    for(j = 0; j < n; j++)
+                        thetas2(ind) += (Theta(j, i) - thetas(i)) * (Theta(j, k) - thetas(k)) *
+                            L(j) * prior(j) / denom;
+                    thetas2(ind) += (thetas(i) - mu(i)) * (thetas(k) - mu(k));
+                    scores2(pat, ind) = thetas2(ind);
+                    ind += 1;
+                }
+            }
+        }
+    }
+
+    List ret;    
+    ret["scores"] = scores;
+    ret["scores2"] = scores2;
+    return(ret);
+
+    END_RCPP
+}
+
+
+
